@@ -1065,9 +1065,12 @@ def _eagerly_encode_cf_datetime(
             # parse with cftime instead
             raise OutOfBoundsDatetime
         assert np.issubdtype(dates.dtype, "datetime64")
-        if calendar in ["standard", "gregorian"] and np.nanmin(dates).astype(
-            "=M8[us]"
-        ).astype(datetime) < datetime(1582, 10, 15):
+        if (
+            calendar in ["standard", "gregorian"]
+            and dates.size > 0
+            and np.nanmin(dates).astype("=M8[us]").astype(datetime)
+            < datetime(1582, 10, 15)
+        ):
             raise_gregorian_proleptic_gregorian_mismatch_error = True
 
         time_unit, ref_date = _unpack_time_unit_and_ref_date(units)
@@ -1416,8 +1419,9 @@ def resolve_time_unit_from_attrs_dtype(
     dtype = np.dtype(attrs_dtype)
     resolution, _ = np.datetime_data(dtype)
     resolution = cast(NPDatetimeUnitOptions, resolution)
+    time_unit: PDDatetimeUnitOptions
     if np.timedelta64(1, resolution) > np.timedelta64(1, "s"):
-        time_unit = cast(PDDatetimeUnitOptions, "s")
+        time_unit = "s"
         message = (
             f"Following pandas, xarray only supports decoding to timedelta64 "
             f"values with a resolution of 's', 'ms', 'us', or 'ns'. Encoded "
@@ -1430,7 +1434,7 @@ def resolve_time_unit_from_attrs_dtype(
         )
         emit_user_level_warning(message)
     elif np.timedelta64(1, resolution) < np.timedelta64(1, "ns"):
-        time_unit = cast(PDDatetimeUnitOptions, "ns")
+        time_unit = "ns"
         message = (
             f"Following pandas, xarray only supports decoding to timedelta64 "
             f"values with a resolution of 's', 'ms', 'us', or 'ns'. Encoded "
@@ -1517,24 +1521,24 @@ class CFTimedeltaCoder(VariableCoder):
                     time_unit = self.time_unit
             else:
                 if self._emit_decode_timedelta_future_warning:
+                    var_string = f"the variable {name!r}" if name else ""
                     emit_user_level_warning(
                         "In a future version, xarray will not decode "
-                        "timedelta values based on the presence of a "
-                        "timedelta-like units attribute by default. Instead "
-                        "it will rely on the presence of a timedelta64 dtype "
-                        "attribute, which is now xarray's default way of "
-                        "encoding timedelta64 values. To continue decoding "
-                        "timedeltas based on the presence of a timedelta-like "
-                        "units attribute, users will need to explicitly "
-                        "opt-in by passing True or "
-                        "CFTimedeltaCoder(decode_via_units=True) to "
-                        "decode_timedelta. To silence this warning, set "
-                        "decode_timedelta to True, False, or a "
-                        "'CFTimedeltaCoder' instance.",
+                        f"{var_string} into a timedelta64 dtype based on the "
+                        "presence of a timedelta-like 'units' attribute by "
+                        "default. Instead it will rely on the presence of a "
+                        "timedelta64 'dtype' attribute, which is now xarray's "
+                        "default way of encoding timedelta64 values.\n"
+                        "To continue decoding into a timedelta64 dtype, either "
+                        "set `decode_timedelta=True` when opening this "
+                        "dataset, or add the attribute "
+                        "`dtype='timedelta64[ns]'` to this variable on disk.\n"
+                        "To opt-in to future behavior, set "
+                        "`decode_timedelta=False`.",
                         FutureWarning,
                     )
                 if self.time_unit is None:
-                    time_unit = cast(PDDatetimeUnitOptions, "ns")
+                    time_unit = "ns"
                 else:
                     time_unit = self.time_unit
 
